<?php

declare(strict_types=1);

namespace Drupal\languagewire_translation_provider\Cron\Job;

use Drupal\languagewire_translation_provider\Adapter\SystemInterface;
use Drupal\languagewire_translation_provider\Client\LanguageWireClientFactory;
use Drupal\languagewire_translation_provider\Database\ProjectRepositoryInterface;
use Drupal\languagewire_translation_provider\Domain\Project;
use Drupal\languagewire_translation_provider\Domain\ProjectStatus;
use Drupal\languagewire_translation_provider\Service\ProjectCreator;
use Drupal\languagewire_translation_provider\Settings\OrderType;

/**
 * Project Creation Job.
 */
final class ProjectCreationJob extends BaseJob {

  /**
   * Project creator.
   *
   * @var \Drupal\languagewire_translation_provider\Domain\ProjectCreator
   */
  private $projectCreator;

  /**
   * Constructs a new ProjectCreationJob object.
   *
   * @param \Drupal\languagewire_translation_provider\Adapter\SystemInterface $system
   *   System.
   * @param \Drupal\languagewire_translation_provider\Database\ProjectRepositoryInterface $projectRepository
   *   Project repository.
   * @param \Drupal\languagewire_translation_provider\Client\LanguageWireClientFactory $clientFactory
   *   LanguageWire client factory.
   * @param \Drupal\languagewire_translation_provider\Domain\ProjectCreator $projectCreator
   *   Project creator.
   */
  public function __construct(
    SystemInterface $system,
    ProjectRepositoryInterface $projectRepository,
    LanguageWireClientFactory $clientFactory,
    ProjectCreator $projectCreator
  ) {
    parent::__construct($system, $projectRepository, $clientFactory);
    $this->projectCreator = $projectCreator;
  }

  /**
   * Get job name.
   */
  protected function jobName(): string {
    return "Project Creation";
  }

  /**
   * Get projects.
   *
   * @return \Drupal\languagewire_translation_provider\Domain\Project[]
   *   Projects.
   */
  protected function getProjects(): array {
    return $this->projectRepository->getAllByStatus(ProjectStatus::underCreation());
  }

  /**
   * Process project.
   */
  protected function processProject(Project $project): void {
    $tmgmtJob = $project->getTmgmtJob();

    $this->projectCreator->createProject(
      $project,
      $this->getLanguageWireClient($tmgmtJob),
      $tmgmtJob->getOrderType() === OrderType::QUOTATION_ORDER
    );
    $this->projectRepository->save($project);
  }

}
